// If this code works, it was written by Fred Hoogduin.
// If not, I don't know who wrote it.
// pqidesc.cpp : Defines the entry point for the DLL application.
//

#include "stdafx.h"
#include "pqidesc.h"


// Some constants used throughout this module

#define AUTHORNAME_c	"Fred Hoogduin"
#define MODULENAME_c	"Drive Image image file description module"
#define VERSION_c		"1.0"

#define PQIPLUGINID_c	0

						// the next scanf format string says: 'read all characters until a binary
						// zero, but not more than 32768'. I didn't see a (simple) way to get
						// MAXDESCLENGTH in here, sorry :)
#define FSCANDEFINITION_c "%32768[^\0]"

// Prototypes of functions local to this module only

static int iDescribePQI( LPIMPORTSAMEFILEREC pImportSameFile );

// Standard DLL entry point to satisfy WIN32

BOOL APIENTRY DllMain( HANDLE hModule, 
                       DWORD  ul_reason_for_call, 
                       LPVOID lpReserved
					 )
{
	// Suppress 'unrefenced ...' messages
	( void ) hModule;
	( void ) ul_reason_for_call;
	( void ) lpReserved;
    switch (ul_reason_for_call)
	{
		case DLL_PROCESS_ATTACH:
		case DLL_THREAD_ATTACH:
		case DLL_THREAD_DETACH:
		case DLL_PROCESS_DETACH:
			break;
    }
    return TRUE;
}

// The Where Is It specific functions

extern "C" PQIDESC_API void __stdcall ModuleInfoEx( LPMODULEINFOREC pModuleInfo )
{
	// Setup a simple plugin information
	strcpy( pModuleInfo->lpszAuthor, AUTHORNAME_c );
	strcpy( pModuleInfo->lpszModuleName, MODULENAME_c );
	strcpy( pModuleInfo->lpszVersion, VERSION_c );
}

extern "C" PQIDESC_API void __stdcall ConfigureProc( HANDLE hInstance, HANDLE hOwnerWnd, WORD wPluginNumber )
{
	// Suppress 'unrefenced ...' messages
	( void ) hInstance;
	( void ) wPluginNumber;
	// Configure whatever there is to configure
	MessageBox( (HWND) hOwnerWnd, "This could be the configuration dialog", "About PQIDesc.DLL", MB_ICONINFORMATION + MB_OK );
}

extern "C" PQIDESC_API void __stdcall RegisterDescPlugins( TREGISTERPLUGINPROC RegisterPluginProc )
{
	// Register each and every plugin implemented in this DLL
	RegisterPluginProc( PQIPLUGINID_c, ptSameFile, "Drive Image Files", "*.PQI", ConfigureProc );
}

extern "C" PQIDESC_API int __stdcall ImportDesc_SameFileEx(WORD wPluginID, LPIMPORTSAMEFILEREC pImportSameFile)
{
	int iRc = -1;
	// Okay dispatch calls for every SameFileEx implemented in this DLL
	switch ( wPluginID ) {
	case PQIPLUGINID_c :
		// Let's describe the file
		iRc = iDescribePQI( pImportSameFile );
		break;
	default :
		// Oops, an unrecognized plugin id
		break;
	}
	return iRc;
}


// The actual 'workhorse' for plugin PQIPLUGINID_c

static int iDescribePQI( LPIMPORTSAMEFILEREC pImportSameFile )
{
	// We need to access the file, so let's ask WII to grant this
	// According to the documentation this call never fails
	LPCTSTR pszFilename = pImportSameFile->lpRequireFileProc( pImportSameFile->lpszFileName );

	// Open the file for reading in binary mode
	FILE *fin = fopen( pszFilename, "rb" );
	if ( fin == NULL ) {
#ifndef NDEBUG
		strcpy( pImportSameFile->lpszDesc, "Can not open: " );
		strcat( pImportSameFile->lpszDesc, pImportSameFile->lpszFileName );
		strcat( pImportSameFile->lpszDesc, " as " ); 
		strcat( pImportSameFile->lpszDesc, pszFilename );
		return 0;
#else
		return -1;
#endif
	}

	// Re-position to the desired offset
	int iRc = fseek( fin, 0x70, SEEK_SET);
	if ( iRc != 0 ) {
#ifndef NDEBUG
		strcpy( pImportSameFile->lpszDesc, "Can not re-position within: " );
		strcat( pImportSameFile->lpszDesc, pImportSameFile->lpszFileName );
		strcat( pImportSameFile->lpszDesc, " as " ); 
		strcat( pImportSameFile->lpszDesc, pszFilename );
		return 0;
#else
		return -1;
#endif
	}

	// Allocate a character buffer, no need to be larger than the WII allowed maximum size
	char *pszDescription = new char[MAXDESCLENGTH];
	if ( pszDescription == NULL ) {
#ifndef NDEBUG
		strcpy( pImportSameFile->lpszDesc, "Can not allocate memory while describing : " );
		strcat( pImportSameFile->lpszDesc, pImportSameFile->lpszFileName );
		strcat( pImportSameFile->lpszDesc, " as " ); 
		strcat( pImportSameFile->lpszDesc, pszFilename );
		return 0;
#else
		return -1;
#endif
	}

	// Read from file until the first binary zero
	iRc = fscanf( fin, FSCANDEFINITION_c, pszDescription );
	if ( iRc != 1 ) {
#ifndef NDEBUG
		strcpy( pImportSameFile->lpszDesc, "Can not read string from : " );
		strcat( pImportSameFile->lpszDesc, pImportSameFile->lpszFileName );
		strcat( pImportSameFile->lpszDesc, " as " ); 
		strcat( pImportSameFile->lpszDesc, pszFilename );
		return 0;
#else
		return -1;
#endif
	}

	// Close the file
	( void ) fclose( fin );

	// Make sure there will allways be a binary zero at the end of the buffer
	pszDescription[MAXDESCLENGTH -1] = '\0';

	// Copy the description
	strcpy( pImportSameFile->lpszDesc, pszDescription );

	// Free allocated memory
	delete [] pszDescription;

	// All went well
	return 0;
}
